@@ -7,9 +7,13 @@ stdEr <- function(x, ...)
 stdEr.default <- function(x, ...) {
    if(!is.null(x$std))
        return(x$std)
-   s <- sqrt(diag(vcov(x)))
-   names(s) <- names(coef(x))
-   s
+   if(!is.null(vc <- vcov(x))) {
+      s <- sqrt(diag(vc))
+      names(s) <- names(coef(x))
+      return(s)
+   }
+   return(NULL)
+                           # if neither std nor vcov is defined, we return NULL...
 }
 
 stdEr.lm <- function(x, ...)
